<?php

declare(strict_types=1);

namespace SuperRhino\Utils\MQClient;

use SuperRhino\Utils\Facade;
use SuperRhino\Utils\MQClient\Exception\MQException;


interface MQClientAdapterInterface {
    /**
     * 生产消息
     * @param string $topic
     * @param string $messageBody
     * @param array $config <p>
     * @see Constants 一些消息配置，
     * MESSAGE_PROPERTIES_MSG_KEY : 设置消息KEY，如果没有设置，则消息的KEY为RequestId
     * MESSAGE_PROPERTIES_TIMER_KEY : 定时消息，单位毫秒（ms），在指定时间戳（当前时间之后）进行投递。
     * MESSAGE_PROPERTIES_TRANS_CHECK_KEY : 在消息属性中添加第一次消息回查的最快时间，单位秒，并且表征这是一条事务消息
     * </p>
     * @return string
     */
    public function produce(string $topic, string $messageBody, array $config = []): string;

    /**
     * 消费消息，注意如果没有消息可以消费，已经做过预处理会返回空数组而不会像官方文档所述报错
     * @param string $topic 话题
     * @param string $groupId 消费者group
     * @param int $numOfMessages 获取消息数量 范围为1-15
     * @param int $waitSeconds <p>
     *     队列无消息可消费时的等待时间。如果传-1时不会等待
     *     比如传了10秒，那么在队列无消息时，请求会等待最多10秒钟，直到队列中插入了消息就会直接返回
     *     取值范围为-1 ~ 30
     * </p>
     * @return Message[]
     * @throws MQException
     */
    public function consume(string $topic, string $groupId, int $numOfMessages, int $waitSeconds = 3): array;

    /**
     * 确认对应receiptHandle的消息已经被消费
     * @see Message::$receiptHandle
     * @param string $topic 话题
     * @param string $groupId 消费者group
     * @param string[] $receiptHandles receiptHandle数组，从Message::getMessage中取出
     * @return bool
     * @throws MQException
     */
    public function ack(string $topic, string $groupId, array $receiptHandles): bool;
    
    
    /**
     * 确认对应消息数组的消息已经被消费
     * @param string $topic 话题
     * @param string $groupId 消费者
     * @param Message[] $messages 从consume中取出的消息数组
     * @return bool
     * @throws MQException
     */
    public function ackMessages(string $topic, string $groupId, array $messages): bool;
}
